# NOT A REAL CONFIG
# Should you want to play around with your own REDIS instance,
# you config might look something like this
REDIS_CONFIG = {"password":"HqtnotarealkeyWkeF",
                "host":"redis-someting-blah.com",
                "port":"35406"}
